﻿// --------------------------------------------------------------------------------------
//
// FRmain.cs                                                      Ulrich Müller  28.03.08
//                                                                PCS1700 C# 2005 Express
// Demo Robot aus Profi Computing 1991 : Lösung unter Nutzung der Klasse FishRobot
//      Die Ansteuerung der Home Position erfolgt hier nur bei Start des Programms
//      Der simultane Ablauf wurde reduziert, da die Tonne dann nicht sicher gegriffen
//      werden kann. Durch schlichtes Weglassen einzelner MoveTo-Methoden-Aufrufe
//      kann die Simultanität - zu Lasten der Betriebssicherheit - aber gesteigert
//      werden.
//      Achtung feste Zuordnung der Taster : 
//              M1 (I1 = Ende, I2 = Impuls), M2 (I3, I4), M3 (I5, I6)
//      Basis ist die Assembly FishFace2005.DLL
//
// --------------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using FishFace40;

namespace Rob91FR {
  public partial class Rob91FR : Form {

    const Out mDreh = Out.M1;
    int DrehPos1;
    int DrehPos2;
    const Out mArm = Out.M2;
    const int cArmOben = 0;
    int ArmUnten;
    const Out mZange = Out.M3;
    const int cZangeOffen = 0;
    int ZangeZu;

    // --- Instanzierung mit Daten für die Industry Robots ---------------
    FishRobot ft = new FishRobot(new int[,] 
                   { { (int)mDreh, 222 }, { (int)mArm, 20 }, { (int)mZange, 30 }});
    private void PositionAusgabe(object sender, int[] actPos) {
      // --- Ereignis-Routine zur Positionsanzeige ----------
      string Pos = "Position : ";
      for (int i = 0; i < actPos.Length - 1; i++) Pos += actPos[i].ToString() + " - ";
      lblPosition.Text = Pos + actPos[actPos.Length - 1];
    }

    private void Action() {
      ZangeZu = (int)numZange.Value;
      ArmUnten = (int)numArm.Value;
      DrehPos1 = (int)numPos1.Value;
      DrehPos2 = (int)numPos2.Value;
      lblStatus.Text = "Fährt auf Home-Position";
      ft.MoveHome();
      do {
        lblStatus.Text = "Fährt auf Position 1";
        ft.MoveTo(DrehPos1);
        ft.MoveTo(DrehPos1, ArmUnten);
        ft.MoveTo(DrehPos1, ArmUnten, ZangeZu);
        ft.MoveTo(DrehPos1, cArmOben);
        lblStatus.Text = "Fährt auf Position 2";
        ft.MoveTo(DrehPos2);
        ft.MoveTo(DrehPos2, ArmUnten);
        ft.MoveTo(DrehPos2, ArmUnten, cZangeOffen);
        ft.MoveTo(DrehPos2, cArmOben);
      } while (!ft.Finish());
    }

    #region --- Programm-Kontrolle ---
    public Rob91FR() {
      InitializeComponent();
    }

    private void cmdAction_Click(object sender, EventArgs e) {
      // --- Open erstes ROBO an USB oder Intelligent Interface an COM
      //     während des Ablaufs von Action() gesperrt ---------------
      ft.PositionChange += new FishRobot.CommonDelegate(PositionAusgabe);
      try {
        if (cboPortName.SelectedIndex == 0)
          ft.OpenInterface(IFTypen.ftROBO_first_USB, 0);
        else ft.OpenInterface(IFTypen.ftIntelligent_IF, cboPortName.SelectedIndex, 5);
        cmdAction.Enabled = false;
        cmdEnde.Text = "&HALT";
        lblStatus.Text = "--- Bei der Arbeit ---";
        lblHinweis.Text = "--- Ende : HALT-Button oder ESC-Taste ---";

        Action();        // --- Haupt-Routine der Anwendung

        ft.ClearMotors();
      }
      catch (FishFaceException eft) {
        MessageBox.Show(eft.Message, this.Text,
          MessageBoxButtons.OK, MessageBoxIcon.Stop);
      }
      finally {
        ft.CloseInterface();
        cmdAction.Enabled = true;
        cmdEnde.Text = "&ENDE";
        lblStatus.Text = "--- Auf ein Neues ---";
        lblHinweis.Text = "Nochmal : ACTION, sonst ENDE oder ESC-Taste";
        cmdAction.Focus();
      }
    }

    private void cmdEnde_Click(object sender, EventArgs e) {
      // --- Programmende nur, wenn Action nicht läuft ---
      if (cmdEnde.Text == "&HALT") ft.NotHalt = true; else this.Close();
    }

    private void formLoad(object sender, EventArgs e) {
      // --- Default-Interface : Erstes ROBO Interface an USB ----
      cboPortName.SelectedIndex = 2;
    }

    private void formClosing(object sender, FormClosingEventArgs e) {
      // --- Wenn Action läuft, wird ein Programmabbruch über (x) unterbunden ------
      if (cmdEnde.Text == "&HALT") e.Cancel = true;
    }
    #endregion
  }
}