﻿// --------------------------------------------------------------------------------------
//
// LLmain.cs                                                      Ulrich Müller  26.03.08
//                                                                PCS1700 C# 2005 Express
// Demoprogramm für der Robot aus Profi Computing 1991 : MultiThread-Lösung
//   Drehkranz, Arm und Zange laufen in eigenen Threads und synchronisieren sich
//   über ManualResetEvents.
//   Achtung : M1 (I1 = Ende, I2 = Impuls), M2 (I3, I4), M3 (I5, I6)
//             Die Drehrichtung der Motoren wurde umgekehrt
//   Basis ist die Assembly FishFace2005.DLL
//
// Ablauf entsprechend Lucky Logic Programm (Endlosschleife bis ESC-Taste) :
// 1. Anfahren Grundstellung
//    simultan : Dreh auf Endtaster, Arm hoch, Zange öffnen
// 2. Anfahren Position 1
//    simultan : Dreh auf Pos 1, Arm nach unten
//    danach   : Zange schließen
// 3. Anfahren Position 2
//    Dreh auf Pos 2 (Arm bleibt unten)
//
// --------------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using FishFace40;

namespace Rob91LL {
  public partial class Rob91LL : Form {
    FishFace ft = new FishFace();
    Thread td, ta, tz;           // --- Threads : Drehkranz, Arm, Zange 
    ManualResetEvent tdEnd, tdPos1, tdPos2, tzAuf, tzZu;

    const Inp eDrehEnd = Inp.I1;
    const Inp eDrehImpuls = Inp.I2;
    const Inp eArmOben = Inp.I3;
    const Inp eArmImpuls = Inp.I4;
    const Inp eZangeAuf = Inp.I5;
    const Inp eZangeImpuls = Inp.I6;

    const Out mDreh = Out.M1;
    const Dir cDrehRechts = Dir.Links;
    const Dir cDrehLinks = Dir.Rechts;
    int DrehPos1 = 112;
    int DrehPos2 = 20;
    const Out mArm = Out.M2;
    const Dir cArmAuf = Dir.Links;
    const Dir cArmAb = Dir.Rechts;
    int ArmUnten = 15;
    const Out mZange = Out.M3;
    const Dir cZangeAuf = Dir.Links;
    const Dir cZangeZu = Dir.Rechts;
    int ZangeZu = 26;

    private void Action() {
      // --- Anlegen und Starten der Threads
      td = new Thread(tdExecute);
      td.IsBackground = true;
      tdEnd = new ManualResetEvent(false);
      tdPos1 = new ManualResetEvent(false);
      tdPos2 = new ManualResetEvent(false);
      td.Start();

      ta = new Thread(taExecute);
      ta.IsBackground = true;
      ta.Start();

      tz = new Thread(tzExecute);
      tz.IsBackground = true;
      tzAuf = new ManualResetEvent(false);
      tzZu = new ManualResetEvent(false);
      tz.Start();

      // --- Aufrechterhalten des HauptThreads ---
      while (!ft.Finish()) ft.Pause(123);
      td.Abort(); ta.Abort(); tz.Abort();
    }

    // --- ThreadRoutine für den Drehkranz ---
    private void tdExecute() {
      do {
        // --- Grundstellung ----
        lblStatus.Text = "Anfahren Grundstellung";
        tdPos2.Reset();
        lblDreh.Text = "Action";
        ft.SetMotor(mDreh, cDrehRechts);
        ft.WaitForInput(eDrehEnd);
        ft.SetMotor(mDreh, Dir.Aus);
        lblDreh.Text = "EndTaster";
        tdEnd.Set();

        // --- Anfahren Position 1 -----
        lblStatus.Text = "Anfahren Position 1";
        tdEnd.Reset();
        lblDreh.Text = "Action";
        ft.SetMotor(mDreh, cDrehLinks);
        ft.WaitForChange(eDrehImpuls, DrehPos1);
        ft.SetMotor(mDreh, Dir.Aus);
        lblDreh.Text = "Position 1";
        tdPos1.Set();
        tzZu.WaitOne();

        // --- Anfahren Position 2 ----
        lblStatus.Text = "Anfahren Position 2";
        tdPos1.Reset();
        lblDreh.Text = "Action";
        ft.SetMotor(mDreh, cDrehRechts);
        ft.WaitForChange(eDrehImpuls, DrehPos2);
        ft.SetMotor(mDreh, Dir.Aus);
        lblDreh.Text = "Position 2";
        tdPos2.Set();
        tzAuf.WaitOne();
      } while (!ft.Finish());
    }
    
    // --- ThreadRoutine des Arms ---
    private void taExecute() {
      do {
        // --- Arm aufwärts ---
        lblArm.Text = "Action";
        ft.SetMotor(mArm, cArmAuf);
        ft.WaitForInput(eArmOben);
        ft.SetMotor(mArm, Dir.Aus);
        lblArm.Text = "Oben";
        tdEnd.WaitOne();

        // --- Arm abwärts ---
        lblArm.Text = "Action";
        ft.SetMotor(mArm, cArmAb);
        ft.WaitForChange(eArmImpuls, ArmUnten);
        ft.SetMotor(mArm, Dir.Aus);
        lblArm.Text = "Unten";
        tdPos1.WaitOne();
        tdPos2.WaitOne();
      } while (!ft.Finish());
    }
    
    // --- ThreadRoutine der Zange ---
    private void tzExecute() {
      do {
        // --- Zange öffnen ---
        lblZange.Text = "Action";
        tzAuf.Reset();
        ft.SetMotor(mZange, cZangeAuf);
        ft.WaitForInput(eZangeAuf);
        ft.SetMotor(mZange, Dir.Aus);
        lblZange.Text = "Offen";
        tzAuf.Set();
        tdPos1.WaitOne();

        // --- Zange schließen ---
        lblZange.Text = "Action";
        tzZu.Reset();
        ft.SetMotor(mZange, cZangeZu);
        ft.WaitForChange(eZangeImpuls, ZangeZu);
        ft.SetMotor(mZange, Dir.Aus);
        lblZange.Text = "Geschlossen";
        tzZu.Set();
        tdPos2.WaitOne();
      } while (!ft.Finish());
    }

    #region --- Programm-Kontrolle ---
    public Rob91LL() {
      InitializeComponent();
    }

    private void cmdAction_Click(object sender, EventArgs e) {
      // --- Open erstes ROBO an USB oder Intelligent Interface an COM
      //     während des Ablaufs von Action() gesperrt ---------------
      try {
        if (cboPortName.SelectedIndex == 0)
          ft.OpenInterface(IFTypen.ftROBO_first_USB, 0);
        else ft.OpenInterface(IFTypen.ftIntelligent_IF, cboPortName.SelectedIndex, 5);
        cmdAction.Enabled = false;
        cmdEnde.Text = "&HALT";
        lblStatus.Text = "--- Bei der Arbeit ---";
        lblHinweis.Text = "--- Ende : HALT-Button oder ESC-Taste ---";

        Action();        // --- Haupt-Routine der Anwendung

        ft.ClearMotors();
      }
      catch (FishFaceException eft) {
        MessageBox.Show(eft.Message, this.Text,
          MessageBoxButtons.OK, MessageBoxIcon.Stop);
      }
      finally {
        ft.CloseInterface();
        cmdAction.Enabled = true;
        cmdEnde.Text = "&ENDE";
        lblStatus.Text = "--- Auf ein Neues ---";
        lblHinweis.Text = "Nochmal : ACTION, sonst ENDE oder ESC-Taste";
        cmdAction.Focus();
      }
    }

    private void cmdEnde_Click(object sender, EventArgs e) {
      // --- Programmende nur, wenn Action nicht läuft ---
      if (cmdEnde.Text == "&HALT") ft.NotHalt = true; else this.Close();
    }

    private void formLoad(object sender, EventArgs e) {
      // --- Default-Interface : Erstes ROBO Interface an USB ----
      cboPortName.SelectedIndex = 2;
      Label.CheckForIllegalCrossThreadCalls = false;
    }

    private void formClosing(object sender, FormClosingEventArgs e) {
      // --- Wenn Action läuft, wird ein Programmabbruch über (x) unterbunden ------
      if (cmdEnde.Text == "&HALT") e.Cancel = true;
    }
    #endregion
  }
}