﻿// --------------------------------------------------------------------------------------
//
// RMmain.cs                                                      Ulrich Müller  27.03.08
//                                                                PCS1700 C# 2005 Express
// Demo Robot aus Profi Computing 1991 : Lösung unter Nutzung von RobMotoren
//      Die Ansteuerung der Home Position erfolgt hier nur bei Start des Programms
//      Der simultane Ablauf wurde auf das Anfahren der Grundstellung beschränkt,
//      da die Tonne sonst nicht sicher gegriffen werden kann.
//      Achtung : M1 (I1 = Ende, I2 = Impuls), M2 (I3, I4), M3 (I5, I6)
//      Basis ist die Assembly FishFace2005.DLL
//
// --------------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using FishFace40;

namespace Rob91RM {
  public partial class Rob91RM : Form {
    FishFace ft = new FishFace();

    const Out mDreh        = Out.M1;
    const Dir cDrehRechts  = Dir.Links;
    const Dir cDrehLinks   = Dir.Rechts;
    const Inp eDrehEnd     = Inp.I1;
    const Inp eDrehImpuls  = Inp.I2;
          int DrehPos1     = 112;
          int DrehPos2     = 20;
          int DrehAktPos   = 0;

    const Out mArm         = Out.M2;
    const Dir cArmAuf      = Dir.Links;
    const Dir cArmAb       = Dir.Rechts;
    const Inp eArmOben     = Inp.I3;
    const Inp eArmImpuls   = Inp.I4;
    const int cArmOben     = 0;
          int ArmUnten     = 15;

    const Out mZange       = Out.M3;
    const Dir cZangeAuf    = Dir.Links;
    const Dir cZangeZu     = Dir.Rechts;
    const Inp eZangeOffen  = Inp.I5;
    const Inp eZangeImpuls = Inp.I6;
    const int cZangeOffen  = 0;
          int ZangeZu      = 26;

    private void Home() {
      lblDreh.Text  ="Action";
      lblArm.Text   = "Action";
      lblZange.Text = "Action";
      ft.SetMotor(mDreh, cDrehRechts, Speed.Full, 999);
      ft.SetMotor(mArm, cArmAuf, Speed.Full, 999);
      ft.SetMotor(mZange, cZangeAuf, Speed.Full, 999);
      ft.WaitForMotors(0,(int)mDreh,(int)mArm,(int)mZange);
      DrehAktPos = 0;
      lblDreh.Text = "Home";
      lblArm.Text = "Oben";
      lblZange.Text = "Offen";
    }
    private void Drehen(int Pos){
      lblDreh.Text = "Action";
      if (Pos < DrehAktPos) ft.SetMotor(mDreh, cDrehRechts, Speed.Full, 999);
      else ft.SetMotor(mDreh, cDrehLinks, Speed.Full, Math.Abs(DrehAktPos - Pos));
      ft.WaitForMotors(0, (int)mDreh);
      DrehAktPos = Pos;
      lblDreh.Text = "Pos : " + Pos.ToString();
    }
    private void Arm(int Pos) {
      lblArm.Text = "Action";
      if (Pos == 0) ft.SetMotor(mArm, cArmAuf, Speed.Full, 999);
      else ft.SetMotor(mArm, cArmAb, Speed.Full, Pos);
      ft.WaitForMotors(0, (int)mArm);
      lblArm.Text = Pos == 0 ? "Oben" : "Unten";
    }
    private void Zange(int Pos) {
      lblZange.Text = "Action";
      if (Pos == 0) ft.SetMotor(mZange, cZangeAuf, Speed.Full, 999);
      else ft.SetMotor(mZange, cZangeZu, Speed.Full, Pos);
      ft.WaitForMotors(0, (int)mZange);
      lblZange.Text = Pos == 0 ? "Offen" : "Geschlossen";
    }
    private void Action() {
      ZangeZu  = (int)numZange.Value;
      ArmUnten = (int)numArm.Value;
      DrehPos1 = (int)numPos1.Value;
      DrehPos2 = (int)numPos2.Value;
      lblStatus.Text = "Fährt auf Grundstellung";
      Home();
      do {
        lblStatus.Text = "Zu Position 1";
        Drehen(DrehPos1);
        Arm(ArmUnten);
        Zange(ZangeZu);
        Arm(cArmOben);
        lblStatus.Text = "Zu Position 2";
        Drehen(DrehPos2);
        Arm(ArmUnten);
        Zange(cZangeOffen);
        Arm(cArmOben);
      } while (!ft.Finish());
    }

    #region --- Programm-Kontrolle ---
    public Rob91RM() {
      InitializeComponent();
    }

    private void cmdAction_Click(object sender, EventArgs e) {
      // --- Open erstes ROBO an USB oder Intelligent Interface an COM
      //     während des Ablaufs von Action() gesperrt ---------------
      try {
        if (cboPortName.SelectedIndex == 0)
          ft.OpenInterface(IFTypen.ftROBO_first_USB, 0);
        else ft.OpenInterface(IFTypen.ftIntelligent_IF, cboPortName.SelectedIndex, 5);
        cmdAction.Enabled = false;
        cmdEnde.Text = "&HALT";
        lblStatus.Text = "--- Bei der Arbeit ---";
        lblHinweis.Text = "--- Ende : HALT-Button oder ESC-Taste ---";

        Action();        // --- Haupt-Routine der Anwendung

        ft.ClearMotors();
      }
      catch (FishFaceException eft) {
        MessageBox.Show(eft.Message, this.Text,
          MessageBoxButtons.OK, MessageBoxIcon.Stop);
      }
      finally {
        ft.CloseInterface();
        cmdAction.Enabled = true;
        cmdEnde.Text = "&ENDE";
        lblStatus.Text = "--- Auf ein Neues ---";
        lblHinweis.Text = "Nochmal : ACTION, sonst ENDE oder ESC-Taste";
        cmdAction.Focus();
      }
    }

    private void cmdEnde_Click(object sender, EventArgs e) {
      // --- Programmende nur, wenn Action nicht läuft ---
      if (cmdEnde.Text == "&HALT") ft.NotHalt = true; else this.Close();
    }

    private void formLoad(object sender, EventArgs e) {
      // --- Default-Interface : Erstes ROBO Interface an USB ----
      cboPortName.SelectedIndex = 2;
    }

    private void formClosing(object sender, FormClosingEventArgs e) {
      // --- Wenn Action läuft, wird ein Programmabbruch über (x) unterbunden ------
      if (cmdEnde.Text == "&HALT") e.Cancel = true;
    }
    #endregion
  }
}